# -*- coding: utf-8 -*-
from globals import *
import random
import copy
import enemy_script
import dialog
import time
import timerepr
import math
import system
import sound
import string
import sys

data = sys.modules["__main__"].data

__doc__= """module for the output of short comments"""

map_initialized = False
mercenary_comments_allowed = False # determines if mercenary comments are allowed

def on_finally_initialized():
    global map_initialized
    map_initialized = True

    # make list of all dealers
    if not hasattr( data(), "traders" ):
        data().traders = []

    for char in system.get_npcs():
        if object.has_attribute( char, "TRADER" ):
            if object.get_attribute( char, "TRADER" ) == "trader":
                data().traders.append( char )

    # make a list of all motorbikes
    possible_motorbikes = [ \
        "RES3D_TRES_OBJECTS_BUILDING_NOMADENBIKE_1",\
        "RES3D_TRES_OBJECTS_BUILDING_NOMADENBIKE_2",\
        "RES3D_TRES_OBJECTS_BUILDING_NOMADENBIKE_3",\
        "RES3D_TRES_OBJECTS_BUILDING_HARLEY",\
        "RES3D_TRES_OBJECTS_BUILDING_MOTOCROSS_BIKE" ]
    if not hasattr( data(), "motorbikes" ):
        data().motorbikes = []

    for item in system.get_objects():
        if object.has_attribute( item, "3d_id" ):
            if object.get_attribute( item, "3d_id" ) in possible_motorbikes:
                data().motorbikes.append( item )

    # make a list of all cars
    possible_cars = [ \
        "RES3D_TRES_OBJECTS_BUILDING_DODGERAM_AUFGEBOCKT",\
        "RES3D_TRES_OBJECTS_BUILDING_DODGERAM",\
        "RES3D_TRES_OBJECTS_BUILDING_CAMARO1",\
        "RES3D_TRES_OBJECTS_BUILDING_CAMARO2",\
        "RES3D_TRES_OBJECTS_BUILDING_DODGERAM2",\
        "RES3D_FLAMMENWERFER",\
        "RES3D_TRES_OBJECTS_BUILDING_HUMVEE_CONVOI_1",\
        "RES3D_TRES_OBJECTS_BUILDING_HUMVEE_CONVOI_2",\
        "RES3D_TRES_OBJECTS_BUILDING_HUMVEE_CONVOI_3",\
        "RES3D_TRES_OBJECTS_BUILDING_HUMVEE_CONVOI_4",\
        "RES3D_TRES_OBJECTS_BUILDING_CONVOY",\
        "RES3D_TRES_OBJECTS_BUILDING_LKW_PRITSCHE" ]
            
    if not hasattr( data(), "cars" ):
        data().cars = []

    for item in system.get_objects():
        if object.has_attribute( item, "3d_id" ):
            if object.get_attribute( item, "3d_id" ) in possible_cars:
                data().cars.append( item )


def on_map_exit( next_zone ):
    global map_initialized
    map_initialized = False
    allow_mercenary_comments( False )
    # empty list of traders
    if hasattr( data(), "traders" ):
        data().traders = []
    # empty list of motorbikes
    if hasattr( data(), "motorbikes" ):
        data().motorbikes = []
    # empty list of cars
    if hasattr( data(), "cars" ):
        data().cars = []

def on_begin_combat():
    None

def on_end_combat():
    dead_npcs = 0
    for npc in system.get_npcs():
        if object.is_destroyed( npc ):
            dead_npcs = dead_npcs + 1

    if dead_npcs >= 2:
        enemy_script.delay( 5, Lambda( 'combat_comments()' ) )

    # set combat end time to measure when to give comments
    data().combat_end_time = system.get_mission_time()

def combat_comments():
    """Makes comments von combats
    """
    mercenary_comment( "MERCENARY_SEVENSOULS", None, "MSEV_COMMENT_C_", 10, "on_combat_won" )
    

def init():
    """initializes the comment system

can be called repeatly and is normally called from comments.comment    
"""
    """
    if "comments" not in globals.data().__dict__:
        globals.data().comments = \
            {
                "MERCENARY_VICTOR" : (
                    "MERCENARY_VICTOR_COMMENT_A_", [1,2,3,4,5]
                                      )
            }
            """

def allow_mercenary_comments( allow ):
    """Mercenary comments should only be allowed if the timer is running.
    Because some comments react on actions that happen on load. 
    These action should not be commented by the mecenaries.
    """
    global mercenary_comments_allowed
    global map_initialized

    if map_initialized:
        old_state = mercenary_comments_allowed
        mercenary_comments_allowed = allow
        if allow == True and old_state == False:
            system.notify_global_timer( "mercenary_party_comment_timer", 600000, True )

def comment( character_id, text ):
    init()
    try:
        raise ValueError
    except ValueError:
        # fallback: if there is no speech for this text show a text panel
        dialog.init_dialog(type=globals.DIALOGSMALL)
        dialog.add_text(text=text, speaker=character_id, follower='ENDDIALOG')
        dialog.start()
    else:
        # if the speech output worked:
        if character_id in globals.data().comments:
            textbase = globals.data().comments[character_id][0]
            variations = globals.data().comments[character_id][1]
    
            if len(variations) > 0:
                choice = random.choice(variations)
                comment(character_id, textbase + str(choice))
                globals.data().comments[character_id][1].remove(choice)

def mercenary_comment( speaker_id, listener_id, comment_id_part, number_of_comments, event ):
    """displays a mercenary comment following all the
    rules for these kind of comments
    Rules are:
    - no comment if under attack
    - no comment another comment/dialog is active
    - no comment if the last comment is less then 10 minutes ago

mercenary_id - id of the mecenary
comment_id_part - id of the comment
number_of_comments - the number of comments that can be selected from
event - name of the event that startet the comment
"""
    # If spoken comments dictionary does not exit, than create it.
    if not hasattr( data(), "spoken_comments" ): 
        data().spoken_comments = {}

    # Test if no other dialog is running.
    # Test if there is no combat running.
    #if not dialog.is_dialog_active() and not system.in_combat_mode():
    if check_global_rules( event ) and check_speaker_listener_rules( speaker_id, listener_id ):
        comment_id = get_unspoken_comment( speaker_id, listener_id, comment_id_part, number_of_comments, event )

        if comment_id != None:
            if not data().spoken_comments.has_key( speaker_id ) or \
            not data().spoken_comments[ speaker_id ].has_key( event ) or\
            data().spoken_comments[ speaker_id ][ event ][ 0 ] == True:

                dialog.init_dialog( type = globals.DIALOGSMALL )    
                dialog.add_text( text = comment_id, speaker = speaker_id, follower = "ENDDIALOG" )
                dialog.start()

                # Comment is spoken, so add it to the spoken comments list
                # so that it is never spoken again
                if not data().spoken_comments.has_key( speaker_id ):
                    data().spoken_comments[ speaker_id ] = {}

                if not data().spoken_comments[ speaker_id ].has_key( event ):
                    data().spoken_comments[ speaker_id ][ event ] = [ False, [] ]

                # Set the event to False, that means a comment of this event could not be
                # spoken until the event is set to True.
                data().spoken_comments[ speaker_id ][ event ][ 0 ] = False
                data().spoken_comments[ speaker_id ][ event ][ 1 ].append( comment_id )

                # party_comment events have their own timer
                if event != "party_comment":
                    # Next comment for this event can be done in 10 minutes.
                    # The timer sets the event to True.
                    timer_id = speaker_id + "_mercenary_comment_timer_" + event
                    if event != "on_combat_won":
                        system.notify_global_timer( timer_id, 600000, False )
                    else:
                        system.notify_global_timer( timer_id, 1800000, False )
                else:
                    data().spoken_comments[ speaker_id ][ event ][ 0 ] = True

                return True

    return False

def check_global_rules( event = False ):
    """Checks for some rules that must be fullfilled
    to speak a comment
    """
    # check if mercenary comments are allowed
    global mercenary_comments_allowed
    if not mercenary_comments_allowed:
        return False

    # check if mercenary comments are activated
    if globals.ACTIVATE_MERCENARY_COMMENTS != 1:
        return False

    # no dialog must be active
    if dialog.is_dialog_active():
        return False
    
    # the party must not be in a combat
    if system.in_combat_mode():
        return False

    # party is not resting
    if hasattr( data(), "rest" ): 
        if data().rest.resting == True:
            return False

    # not cutscene is played
    if system.is_cutscene():
        return False

    # last combat must be 1 realtime minutes ago
    if event != "on_combat_won":
        if hasattr( data(), "combat_end_time" ):
            # das sollte nach dem fix von alex nicht mehr gebraucht werden
            #if data().combat_end_time >system.get_mission_time() :
            #    data().combat_end_time = system.get_mission_time() #jan:  reset after load
            if ( system.get_mission_time() - data().combat_end_time ) / int( globals.hour_game_to_real ) < timerepr.time( "00:01:00" ):
                return False

    return True

def check_speaker_listener_rules( speaker_id, listener_id ):
    """Checks if speaker and listener are follwing given rules
    """
    # speaker and listener must be part of the party
    all_pcs = system.get_pcs()
    if speaker_id not in all_pcs:
        return False

    # speaker and listener must be alive
    if object.is_unconscious( speaker_id ) or object.is_destroyed( speaker_id ):
        return False

    # speaker and listener must not be fighting
    if object.is_fighting( speaker_id ):
        return False

    if listener_id:
        if listener_id not in all_pcs:
            return False

        if object.is_unconscious( listener_id ) or object.is_destroyed( listener_id ):
            return False

        # speaker and listener must be at least 10 meters away
        if math.hypot( object.get_x( speaker_id ) - object.get_x( listener_id ), object.get_y( speaker_id ) - object.get_y( listener_id ) ) > 10:
            return False

        # speaker and listener must be on the same grid
        if object.get_grid( speaker_id ) != object.get_grid( listener_id ):
            return False

        if object.is_fighting( listener_id ):
            return False

    return True

def get_unspoken_comment( speaker_id, listener_id, comment_id_part, number_of_comments, event ):
    print 'get_unspoken_comment(', speaker_id, listener_id, comment_id_part, number_of_comments, event ,')'
    """check for spoken comments and return a random unspoken one.

mercenary_id         - id of the mercenary the should speak a comment
comment_id_part     - the first part of the comment id. the second part is dynamically
                added.
number_of_comments     - number of comments with that id part.
event -            - event that the comment is of
"""
    # make a list of all possible ids for that comment.
    # id is made of comment_id_part and a number
    possible_comments = []
    for x in range( number_of_comments ):
        comment_id = comment_id_part + str( x + 1 )
        if listener_id != None:
            comment_id = comment_id + string.replace( listener_id, "MERCENARY", "", 1 )
        possible_comments.append( comment_id )
    
    # if spoken comments dictionary does not exit, than create it.
    if not hasattr( data(), "spoken_comments" ): 
        data().spoken_comments = {}
    
    spoken_comments = data().spoken_comments

    speakable_comments = copy.copy( possible_comments )
    # remove all spoken comments from the list of possible comments
    if spoken_comments.has_key( speaker_id ):
        speaker = spoken_comments[ speaker_id ]
        # get the comments for this event
        if speaker.has_key( event ):
            event_comments = speaker[ event ][ 1 ]
            for comment in possible_comments:
                if comment in event_comments:
                    speakable_comments.remove( comment )

    if len( speakable_comments ) > 0:
        return random.choice( speakable_comments )
    else:
        return None


############################
### event based comments ###
############################
def on_equipment_add( character_id, equipment_id ):
    """displays mercenary comments if a item is added to the inventory of a mercenary

character_id - id of the char who gets the item
equipment_id - id of the item he got
    """
    # first check if character_id is in party.
    # if not, then return
    # and check for ALTER_EGO. He is also not talked to.
    if character_id not in system.get_pcs() or character_id == "ALTER_EGO":
        return

    if object.has_attribute( equipment_id, "typeid" ):
        equipment_id = object.get_attribute( equipment_id, "typeid" )

    def get_weapon_type( equipment_id ):
        """Returns the weapons type. If the equipment is not 
        a weapon then it returns None"""
        if object.has_attribute( equipment_id, "item_type" ):
            if object.get_attribute( equipment_id, "item_type" ) == "weapon":
                if object.has_attribute( equipment_id, "weapon_type" ):
                    return object.get_attribute( equipment_id, "weapon_type" )

        return None

    ###################################################################
    # if meat is added to the inventory of a mercenary other than agnes.
    ###################################################################
    meat = [ 'SET_MEAT_RAW_HUMONGOUS', 'SET_MEAT_RAW_BIG', 'SET_MEAT_RAW', 'SET_MEAT_RAW_SMALL', 'SET_MEAT_RAW_TINY',\
        'SET_MEAT_COOKED_HUMONGOUS', 'SET_MEAT_COOKED_BIG', 'SET_MEAT_COOKED', 'SET_MEAT_COOKED_SMALL', 'SET_MEAT_COOKED_TINY' ]
    # use string.find to find also cheated chars for testing reasons, because cheated chars
    # sometimes are named mecenary_agnes_d.
    # check if added equipment is meat and if the char that gets the item is not agnes
    if equipment_id in meat and string.find( "MERCENARY_AGNES", character_id ) == -1:
        return mercenary_comment( "MERCENARY_AGNES", character_id, "MAGN_COMMENT_B_", 5, "on_equipment_add" )
        
    ###################################################################
    # if carmen gets a complex mechanical or electrical item
    ###################################################################
    items = [ "SET_NIGHT_VISION_GEAR", "SET_GRAPHICS_CARD", "SET_GAME_JOY", "SET_ELECTRIC_KETTLE",\
            "SET_COMUTER_KEYBOARD", "SET_BROKEN_POCKET_CALCULATOR", "SET_PLIX_COMPENSATOR",\
            "SET_BUG", "SET_WALKIE_TALKIE", "SET_GENERATOR_CONTROL_PANEL", \
            "SET_CARBURATOR", "SET_DISTRIBUTOR", "SET_CORTICAL_INTERFACE", \
            "SET_NANO_BOT_SURGICAL_DEVICE", "SET_THEODOLIT", "SET_CAR_EXHAUST_PIPE", \
            "SET_LASERPOINTER", "SET_TRANSFORMER", "SET_ELECTROSHOCKER" ]

    if equipment_id in items and string.find( "MERCENARY_CARMEN", character_id ) != -1:
        return mercenary_comment( character_id, None, "MCAR_COMMENT_B_", 5, "on_equipment_add" )
    
    ###################################################################
    # if ivana gets a heavy weapon 
    ###################################################################
    if get_weapon_type( equipment_id ) == "schwer" and string.find( "MERCENARY_IVANA", character_id ) != -1:
        return mercenary_comment( character_id, None, "MIVA_COMMENT_A_", 5, "on_equipment_add" )

    ###################################################################
    # if henry gets a heavy weapon 
    ###################################################################
    if get_weapon_type( equipment_id ) == "schwer" and string.find( "MERCENARY_HENRY", character_id ) != -1:
        return mercenary_comment( character_id, None, "MHEN_COMMENT_A_", 5, "on_equipment_add" )

    ###################################################################
    # if sally/lin gets a special throwing weapon
    ###################################################################
    items = [ "SET_SHURIKEN", "SET_BOOMERANG" ]

    if equipment_id in items and string.find( "MERCENARY_SALLY", character_id ) != -1:
        return mercenary_comment( character_id, None, "MSAL_COMMENT_C_", 5, "on_equipment_add" )
    
    ###################################################################
    # if butch gets a heavy weapon
    ###################################################################
    if get_weapon_type( equipment_id ) == "schwer" and string.find( "MERCENARY_BUTCH", character_id ) != -1:
        return mercenary_comment( character_id, None, "MBUT_COMMENT_A_", 5, "on_equipment_add" )

    ###################################################################
    # if sand gets food
    ###################################################################
    items = [ "SET_MEAT_RAW_TINY", "SET_MEAT_RAW_SMALL", "SET_MEAT_RAW", "SET_MEAT_RAW_BIG", "SET_MEAT_RAW_HUMONGOUS", "SET_MEAT_COOKED_TINY",
        "SET_MEAT_COOKED_SMALL", "SET_MEAT_COOKED", "SET_MEAT_COOKED_BIG", "SET_MEAT_COOKED_HUMONGOUS", "SET_CHEESE", "SET_BREAD",
        "SET_APPLE", "SET_CORN_COB", "SET_CARROT" ]

    if equipment_id in items and string.find( "MERCENARY_SAND", character_id ) != -1:
        return mercenary_comment( character_id, None, "MSAN_COMMENT_A_", 10, "on_equipment_add" )

    ###################################################################
    # if storm gets a item that can be combined with another item
    ###################################################################
    if object.has_attribute( equipment_id, "item_combination" ) and string.find( "MERCENARY_STORM", character_id ) != -1:
        return mercenary_comment( character_id, None, "MSTO_COMMENT_A_", 8, "on_equipment_add" )

    ###################################################################
    # if francis gets a item that can be combined with another item
    ###################################################################
    if object.has_attribute( equipment_id, "item_combination" ) and string.find( "MERCENARY_FRANCIS", character_id ) != -1:
        return mercenary_comment( character_id, None, "MFRA_COMMENT_C_", 5, "on_equipment_add" )

    ###################################################################
    # if francis gets a heavy weapon
    ###################################################################
    if get_weapon_type( equipment_id ) == "schwer" and string.find( "MERCENARY_FRANCIS", character_id ) != -1:
        return mercenary_comment( character_id, None, "MFRA_COMMENT_D_", 5, "on_equipment_add" )

    ###################################################################
    # if deborah gets some food
    ###################################################################
    items = [ "SET_MEAT_RAW_TINY", "SET_MEAT_RAW_SMALL", "SET_MEAT_RAW", "SET_MEAT_RAW_BIG", "SET_MEAT_RAW_HUMONGOUS", "SET_MEAT_COOKED_TINY",
        "SET_MEAT_COOKED_SMALL", "SET_MEAT_COOKED", "SET_MEAT_COOKED_BIG", "SET_MEAT_COOKED_HUMONGOUS", "SET_CHEESE", "SET_BREAD",
        "SET_APPLE", "SET_CORN_COB", "SET_CARROT" ]
    

    if equipment_id in items and string.find( "MERCENARY_DEBORAH", character_id ) != -1:
        return mercenary_comment( character_id, None, "MDEB_COMMENT_A_", 5, "on_equipment_add" )

    ###################################################################
    # if victor gets explosives
    ###################################################################
    if get_weapon_type( equipment_id ) == "explosiv" and string.find( "MERCENARY_VICTOR", character_id ) != -1:
        return mercenary_comment( character_id, None, "MVIC_COMMENT_A_", 5, "on_equipment_add" )

    ###################################################################
    # if Keno gets a sniper rifle that he hadn't before
    ###################################################################
    if get_weapon_type( equipment_id ) == "sniper" and string.find( "MERCENARY_KENO", character_id ) != -1:
        owned_snipers = []
        if object.has_attribute( "MERCENARY_KENO", "owned_snipers" ):
            owned_snipers = object.get_attribute( "MERCENARY_KENO", "owned_snipers" )
        
        if not equipment_id in owned_snipers:
            owned_snipers.append( equipment_id )
            object.set_attribute( "MERCENARY_KENO", "owned_snipers", owned_snipers )
            return mercenary_comment( character_id, None, "MKEN_COMMENT_A_", 5, "on_equipment_add" )

def check_for_traders_in_sight():
    """Checks if a mercenary sees a trader
    """
    if not hasattr( data(), "traders" ):
        data().traders = []

    traders = data().traders
    all_pcs = system.get_pcs()

    mercenaries = { "MERCENARY_IVANA": [ "MIVA_COMMENT_B_", 5 ],
            "MERCENARY_CARMEN": [ "MCAR_COMMENT_A_", 5 ] }

    for mercenary in mercenaries.keys():
        if mercenary in all_pcs:
            mercenary_x = object.get_x( mercenary )
            mercenary_y = object.get_y( mercenary )
            mercenary_grid = object.get_grid( mercenary )

            for trader in traders:
                if object.get_grid( trader ) == mercenary_grid:
                    trader_x = object.get_x( trader )
                    trader_y = object.get_y( trader )
                    if math.hypot( mercenary_x - trader_x, mercenary_y - trader_y ) < 15:
                        mercenary_comment( mercenary, None, mercenaries[ mercenary ][ 0 ], mercenaries[ mercenary ][ 1 ], "on_trader_in_sight" )
                        return

def check_for_motorbikes_in_sight():
    """Checks if a motorbike is in the sight of a mercenary
    """
    if not hasattr( data(), "motorbikes" ):
        data().motorbikes = []

    motorbikes = data().motorbikes
    all_pcs = system.get_pcs()

    mercenaries = { "MERCENARY_VASQUEZ": [ "MVAS_COMMENT_A_", 5 ] }

    for mercenary in mercenaries.keys():
        if mercenary in all_pcs:
            mercenary_x = object.get_x( mercenary )
            mercenary_y = object.get_y( mercenary )
            mercenary_grid = object.get_grid( mercenary )

            for motorbike in motorbikes:
                if object.get_grid( motorbike ) == mercenary_grid:
                    motorbike_x = object.get_x( motorbike )
                    motorbike_y = object.get_y( motorbike )
                    if math.hypot( mercenary_x - motorbike_x, mercenary_y - motorbike_y ) < 5:
                        mercenary_comment( mercenary, None, mercenaries[ mercenary ][ 0 ], mercenaries[ mercenary ][ 1 ], "on_motorbike_in_sight" )
                        return

def check_for_cars_in_sight():
    """Checks if a car is in the sight of a mercenary
    """
    if not hasattr( data(), "cars" ):
        data().cars = []

    cars = data().cars
    all_pcs = system.get_pcs()

    mercenaries = { "MERCENARY_BUTCH": [ "MBUT_COMMENT_B_", 5 ] }

    for mercenary in mercenaries.keys():
        if mercenary in all_pcs:
            mercenary_x = object.get_x( mercenary )
            mercenary_y = object.get_y( mercenary )
            mercenary_grid = object.get_grid( mercenary )

            for car in cars:
                if object.get_grid( car ) == mercenary_grid:
                    car_x = object.get_x( car )
                    car_y = object.get_y( car )
                    if math.hypot( mercenary_x - car_x, mercenary_y - car_y ) < 5 and object.is_active(car):
                        mercenary_comment( mercenary, None, mercenaries[ mercenary ][ 0 ], mercenaries[ mercenary ][ 1 ], "on_car_in_sight" )
                        return

def check_for_city():
    """Checks if a specific mercanery reaches a city
    """
    if "MERCENARY_SALLY" not in system.get_pcs():
        return

    if object.is_unconscious( "MERCENARY_SALLY" ) or object.is_destroyed( "MERCENARY_SALLY" ):
        return

    # calc distance of Lin to a city middlepoint
    middlepoints = { "zone2": ( 647.554, 373.328 ),
    "zone_3": ( 497,728 ),
    "zone_4": ( 663.301, 516.527 ),
    "zone_5": ( 302,715 ),
    "zone_6": ( 546,290 ),
    "zone_8": ( 666,265 ),
    "zzone_4": ( 466,432 ),
    "zzone_5": ( 560,272 ),
    "sqzone_3": ( 408,332 ),
    "sqzone_6": ( 335,371 ) }

    zone = string.split( system.get_act_mission_name(), "." )[0]

    approache_cities = []
    if object.has_attribute( "MERCENARY_SALLY", "approached_cities" ):
        approached_cities = object.get_attribute( "MERCENARY_SALLY", "approached_cities" )

    # sally did approach this city before
    if zone in approached_cities:
        return
        
    zone_x = middlepoints[ zone ][0]
    zone_y = middlepoints[ zone ][1]

    sally_x = object.get_x( "MERCENARY_SALLY" )
    sally_y = object.get_y( "MERCENARY_SALLY" )

    if math.hypot( sally_x - zone_x, sally_y - zone_y ) <= 100:
        mercenary_comment( "MERCENARY_SALLY", None, "MSAL_COMMENT_B_", 5, "on_approach_city" )
        approached_cities.append( zone )
        object.set_attributes( "MERCENARY_SALLY", "approached_cities", approache_cities )
    

def check_for_traveled_distance():
    """Checks for the traveled distance and 
    comments it
    """
    if "MERCENARY_MILTON" not in system.get_pcs( True ):
        return
        
    if character.get_body_mode( "MERCENARY_MILTON" ) != "STAND":
        reset_travel_params( "MERCENARY_MILTON" )

    if object.has_attribute( "MERCENARY_MILTON", "travel_start_time" ):
        # if milton is on the road for more than 30 hours
        # then we measure the distance
        if object.get_attribute( "MERCENARY_MILTON", "travel_start_time" ) - system.get_mission_time() >= timerepr.time( "30:00:00" ):
            start_pos = object.get_attribute( "MERCENARY_MILTON", "travel_start_pos" )
            distance = math.hypot( object.get_x( "MERCENARY_MILTON" ) - pos[0], object.get_y( "MERCENARY_MILTON" ) - pos[1] )

            if distance >= 1000:

                mercenary_comment( "MERCENARY_MILTON", None, "MMIL_COMMENT_A_", 5, "on_long_travel" )
                reset_travel_params( "MERCENARY_MILTON" )


def reset_travel_params( character_id ):
    """Resets all parameters neede to measure travel time and distance
    """
    object.set_attribute( character_id, "travel_start_time", system.get_mission_time() )
    object.set_attribute( character_id, "travel_start_pos", ( object.get_x( character_id ), object.get_y( character_id ) ) )
    

def check_for_injury_comments():
    """Checks if a party member is injured and comments it.
    """
    all_pcs = system.get_pcs( True ) # get all pcs including the unconscious ones
    if "ALTER_EGO" in all_pcs:
        all_pcs.remove( "ALTER_EGO" ) # remove ALTER_EGO from list
    random.shuffle( all_pcs )

    for char in all_pcs:
        hitpoints = object.get_attribute( char, "hitpoints" )
        max_hitpoints = object.get_attribute( char, "maxhitpoints" )

        if max_hitpoints == 0: continue
        percent_hitpoints = hitpoints / max_hitpoints

        if percent_hitpoints <= 0.2: # if the char has less then 20% hitpoints
            if "MERCENARY_AGNES" in all_pcs and char != "MERCENARY_AGNES":
                mercenary_comment( "MERCENARY_AGNES", char, "MAGN_COMMENT_A_", 4, "injury_comment" )
                break

        if percent_hitpoints <= 0.1: # if the char has less then 10% hitpoints
            if "MERCENARY_SEVENSOULS" in all_pcs and char != "MERCENARY_SEVENSOULS":
                mercenary_comment( "MERCENARY_SEVENSOULS", char, "MSEV_COMMENT_A_", 5, "injury_comment" )
                break


def party_dialog():
    """Two party member have a dialog.
    They but but together in the party for at least 2 hours
    and maximum 6 hours. Realtime.
    """
    dialog_partners = [  ( "MERCENARY_AGNES", "MERCENARY_VASQUEZ" ), 
                         ( "MERCENARY_CARMEN", "MERCENARY_AGNES" ), 
                         ( "MERCENARY_IVANA", "MERCENARY_BUTCH" ),
                         ( "MERCENARY_HENRY", "MERCENARY_AGNES", "MERCENARY_VASQUEZ", "MERCENARY_VICTOR", "MERCENARY_STORM", 
                           "MERCENARY_SAND", "MERCENARY_BUTCH", "MERCENARY_CARMEN", "MERCENARY_SALLY", "MERCENARY_FRANCIS", "MERCENARY_IVANA",
                           "MERCENARY_SEVENSOULS", "MERCENARY_DEBORAH", "MERCENARY_KENO", "MERCENARY_MILTON" ),
                         ( "MERCENARY_BUTCH", "MERCENARY_SAND" ), 
                         ( "MERCENARY_BUTCH", "MERCENARY_STORM" ),
                         ( "MERCENARY_STORM", "MERCENARY_SAND" ), 
                         ( "MERCENARY_SAND", "MERCENARY_STORM" ),
                         ( "MERCENARY_SEVENSOULS", "MERCENARY_MILTON" ), 
                         ( "MERCENARY_KENO", "MERCENARY_AGNES" ),
                         ( "MERCENARY_VICTOR", "MERCENARY_KENO" ), 
                         ( "MERCENARY_DEBORAH", "MERCENARY_STORM" ),
                         ( "MERCENARY_DEBORAH", "MERCENARY_SAND" )]
                         
    random.shuffle( dialog_partners )

    if not hasattr( data(), "spoken_party_dialogs" ): 
        data().spoken_party_dialogs = {}

    all_pcs = system.get_pcs()

    spoken_party_dialogs = data().spoken_party_dialogs

    for partners in dialog_partners:
        if len( partners ) == 2:
            if partners[ 0 ] not in all_pcs or partners[ 1 ] not in all_pcs:
                continue
            # if speaker and listener do not follow some rules
            if not check_speaker_listener_rules( partners[ 0 ], partners[ 1 ] ):
                continue
        if check_long_enough_in_party( partners ) == True:
            char1 = partners[ 0 ]
            char2 = partners[ 1 ]

            if char1 == "MERCENARY_AGNES":
                if char2 == "MERCENARY_VASQUEZ":
                    if not spoken_party_dialogs.has_key( "AGNES_VASQUEZ" ) or spoken_party_dialogs[ "AGNES_VASQUEZ" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MAGN_VASQUEZ_D_1", speaker = "MERCENARY_AGNES", follower = "MAGN_VASQUEZ_D_2" )
                            dialog.add_text( text = "MAGN_VASQUEZ_D_2", speaker = "MERCENARY_VASQUEZ", follower = "MAGN_VASQUEZ_D_3" )
                            dialog.add_text( text = "MAGN_VASQUEZ_D_3", speaker = "MERCENARY_AGNES", follower = "ENDDIALOG" )
                            dialog.start()
                        
                            spoken_party_dialogs[ "AGNES_VASQUEZ" ] = False
                            return True
            if char1 == "MERCENARY_CARMEN":
                if char2 == "MERCENARY_AGNES":
                    if not spoken_party_dialogs.has_key( "CARMEN_AGNES" ) or spoken_party_dialogs[ "CARMEN_AGNES" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MCAR_AGNES_D_1", speaker = "MERCENARY_CARMEN", follower = "MCAR_AGNES_D_2" )
                            dialog.add_text( text = "MCAR_AGNES_D_2", speaker = "MERCENARY_AGNES", follower = "MCAR_AGNES_D_3" )
                            dialog.add_text( text = "MCAR_AGNES_D_3", speaker = "MERCENARY_CARMEN", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "CARMEN_AGNES" ] = False
                            return True
            if char1 == "MERCENARY_IVANA":
                if char2 == "MERCENARY_BUTCH":
                    if not spoken_party_dialogs.has_key( "IVANA_BUTCH" ) or spoken_party_dialogs[ "IVANA_BUTCH" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MIVA_BUTCH_D_1", speaker = "MERCENARY_IVANA", follower = "MIVA_BUTCH_D_2" )
                            dialog.add_text( text = "MIVA_BUTCH_D_2", speaker = "MERCENARY_BUTCH", follower = "MIVA_BUTCH_D_3" )
                            dialog.add_text( text = "MIVA_BUTCH_D_3", speaker = "MERCENARY_IVANA", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "IVANA_BUTCH" ] = False
                            return True
            if char1 == "MERCENARY_HENRY" and char1 in all_pcs:
                if not spoken_party_dialogs.has_key( "HENRY_ALL" ) or spoken_party_dialogs[ "HENRY_ALL" ] == True:
                    
                    if not dialog.is_dialog_active() and not system.in_combat_mode():
                        dialog.init_dialog( type = globals.DIALOGSMALL )    
                        dialog.add_text( text = "MHEN_PARTY_T_001", speaker = "MERCENARY_HENRY", follower = "ENDDIALOG", callback = party_to_henry )
                        dialog.start()
                            
                        spoken_party_dialogs[ "HENRY_ALL" ] = False
                        return True
            if char1 == "MERCENARY_BUTCH":
                if char2 == "MERCENARY_STORM":
                    if not spoken_party_dialogs.has_key( "BUTCH_STORM" ) or spoken_party_dialogs[ "BUTCH_STORM" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MBUT_STORM_D_1", speaker = "MERCENARY_BUTCH", follower = "MBUT_STORM_D_2" )
                            dialog.add_text( text = "MBUT_STORM_D_2", speaker = "MERCENARY_STORM", follower = "MBUT_STORM_D_3" )
                            dialog.add_text( text = "MBUT_STORM_D_3", speaker = "MERCENARY_BUTCH", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "BUTCH_STORM" ] = False
                            return True
                if char2 == "MERCENARY_SAND":
                    if not spoken_party_dialogs.has_key( "BUTCH_SAND" ) or spoken_party_dialogs[ "BUTCH_SAND" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MBUT_SAND_D_1", speaker = "MERCENARY_BUTCH", follower = "MBUT_SAND_D_2" )
                            dialog.add_text( text = "MBUT_SAND_D_2", speaker = "MERCENARY_SAND", follower = "MBUT_SAND_D_3" )
                            dialog.add_text( text = "MBUT_SAND_D_3", speaker = "MERCENARY_BUTCH", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "BUTCH_SAND" ] = False
                            return True
            if char1 == "MERCENARY_SAND":
                if char2 == "MERCENARY_STORM":
                    if not spoken_party_dialogs.has_key( "SAND_STORM" ) or spoken_party_dialogs[ "SAND_STORM" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MSAN_STORM_D_1", speaker = "MERCENARY_SAND", follower = "MSAN_STORM_D_2" )
                            dialog.add_text( text = "MSAN_STORM_D_2", speaker = "MERCENARY_STORM", follower = "MSAN_STORM_D_3" )
                            dialog.add_text( text = "MSAN_STORM_D_3", speaker = "MERCENARY_SAND", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "SAND_STORM" ] = False
                            return True
            if char1 == "MERCENARY_STORM":
                if char2 == "MERCENARY_SAND":
                    if not spoken_party_dialogs.has_key( "STORM_SAND" ) or spoken_party_dialogs[ "STORM_SAND" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MSTO_SAND_D_1", speaker = "MERCENARY_STORM", follower = "MSTO_SAND_D_2" )
                            dialog.add_text( text = "MSTO_SAND_D_2", speaker = "MERCENARY_SAND", follower = "MSTO_SAND_D_3" )
                            dialog.add_text( text = "MSTO_SAND_D_3", speaker = "MERCENARY_STORM", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "STORM_SAND" ] = False
                            return True
            if char1 == "MERCENARY_SEVENSOULS":
                if char2 == "MERCENARY_MILTON":
                    if not spoken_party_dialogs.has_key( "SEVENSOULS_MILTON" ) or spoken_party_dialogs[ "SEVENSOULS_MILTON" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MSEV_MILTON_D_1", speaker = "MERCENARY_SEVENSOULS", follower = "MSEV_MILTON_D_2" )
                            dialog.add_text( text = "MSEV_MILTON_D_2", speaker = "MERCENARY_MILTON", follower = "MSEV_MILTON_D_3" )
                            dialog.add_text( text = "MSEV_MILTON_D_3", speaker = "MERCENARY_SEVENSOULS", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "SEVENSOULS_MILTON" ] = False
                            return True
            if char1 == "MERCENARY_KENO":
                if char2 == "MERCENARY_AGNES":
                    if not spoken_party_dialogs.has_key( "KENO_AGNES" ) or spoken_party_dialogs[ "KENO_AGNES" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MKEN_AGNES_D_1", speaker = "MERCENARY_KENO", follower = "MKEN_AGNES_D_2" )
                            dialog.add_text( text = "MKEN_AGNES_D_2", speaker = "MERCENARY_AGNES", follower = "MKEN_AGNES_D_3" )
                            dialog.add_text( text = "MKEN_AGNES_D_3", speaker = "MERCENARY_KENO", follower = "MKEN_AGNES_D_4" )
                            dialog.add_text( text = "MKEN_AGNES_D_4", speaker = "MERCENARY_AGNES", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "KENO_AGNES" ] = False
                            return True
            if char1 == "MERCENARY_FRANCIS":
                if char2 == "MERCENARY_KENO":
                    if not spoken_party_dialogs.has_key( "FRANCIS_STORM" ) or spoken_party_dialogs[ "FRANCIS_STORM" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MFRA_KENO_D_1", speaker = "MERCENARY_FRANCIS", follower = "MFRA_KENO_D_2" )
                            dialog.add_text( text = "MFRA_KENO_D_2", speaker = "MERCENARY_KENO", follower = "MFRA_KENO_D_3" )
                            dialog.add_text( text = "MFRA_KENO_D_3", speaker = "MERCENARY_FRANCIS", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "FRANCIS_KENO" ] = False
                            return True
            if char1 == "MERCENARY_DEBORAH":
                if char2 == "MERCENARY_STORM":
                    if not spoken_party_dialogs.has_key( "DEBORAH_STORM" ) or spoken_party_dialogs[ "DEBORAH_STORM" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MDEB_STORM_D_1", speaker = "MERCENARY_DEBORAH", follower = "MDEB_STORM_D_2" )
                            dialog.add_text( text = "MDEB_STORM_D_2", speaker = "MERCENARY_STORM", follower = "MDEB_STORM_D_3" )
                            dialog.add_text( text = "MDEB_STORM_D_3", speaker = "MERCENARY_DEBORAH", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "DEBORAH_STORM" ] = False
                            return True
                if char2 == "MERCENARY_SAND":
                    if not spoken_party_dialogs.has_key( "DEBORAH_SAND" ) or spoken_party_dialogs[ "DEBORAH_SAND" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MDEB_SAND_D_1", speaker = "MERCENARY_DEBORAH", follower = "MDEB_SAND_D_2" )
                            dialog.add_text( text = "MDEB_SAND_D_2", speaker = "MERCENARY_SAND", follower = "MDEB_SAND_D_3" )
                            dialog.add_text( text = "MDEB_SAND_D_3", speaker = "MERCENARY_DEBORAH", follower = "ENDDIALOG" )
                            dialog.start()

                            spoken_party_dialogs[ "DEBORAH_SAND" ] = False
                            return True
            if char1 == "MERCENARY_VICTOR":
                if char2 == "MERCENARY_KENO":
                    if not spoken_party_dialogs.has_key( "VICTOR_KENO" ) or spoken_party_dialogs[ "VICTOR_KENO" ] == True:
                        if not dialog.is_dialog_active() and not system.in_combat_mode():
                            dialog.init_dialog( type = globals.DIALOGSMALL )    
                            dialog.add_text( text = "MVIC_KENO_D_1", speaker = "MERCENARY_VICTOR", follower = "MVIC_KENO_D_2" )
                            dialog.add_text( text = "MVIC_KENO_D_2", speaker = "MERCENARY_KENO", follower = "MVIC_KENO_D_3" )
                            dialog.add_text( text = "MVIC_KENO_D_3", speaker = "MERCENARY_VICTOR", follower = "ENDDIALOG" )
                            dialog.start()
                            spoken_party_dialogs[ "VICTOR_KENO" ] = False
                            return True
    return False

def party_to_henry():
    """
    """
    all_pcs = system.get_pcs()
    # remove alter_ego from list
    if "ALTER_EGO" in all_pcs: all_pcs.remove( "ALTER_EGO" )
    if "MERCENARY_HENRY" in all_pcs: all_pcs.remove( "MERCENARY_HENRY" )

    random.shuffle( all_pcs )
    max_speaker = 3
    if len( all_pcs ) < 3:
        max_speaker = len( all_pcs )

    for x in range( max_speaker ):
        if x == max_speaker - 1: # the last speaker
            lambda_string = 'partymeber_to_henry("' + all_pcs[ x ] + '", True )'
        else:
            lambda_string = 'partymeber_to_henry("' + all_pcs[ x ] + '", False )'

        enemy_script.delay( x + 1, Lambda( lambda_string ) )

def partymeber_to_henry( speaker_id, last ):
    """is called for every member that shous at henry
    """
    if last:
        dialog.init_dialog( type = globals.DIALOGSMALL )
        dialog.add_text( text = "MERCENARY_HENRY_PARTY_T_002", speaker = speaker_id, follower = "MHEN_PARTY_T_002" )
        dialog.add_text( text = "MHEN_PARTY_T_002", speaker = "MERCENARY_HENRY", follower = "ENDDIALOG" )
        dialog.start()
    else:
        dialog.init_dialog( type = globals.DIALOGSMALL )
        dialog.add_text( text = "MERCENARY_HENRY_PARTY_T_002", speaker = speaker_id, follower = "ENDDIALOG" )
        dialog.start()


def check_long_enough_in_party( partners ):
    """Checks if the partners are together in the party for at least 2 hours
    and max 6 hours realtime. returns True.

partners - tuple of two character_id's
"""
    if len( partners ) < 1: return False

    join_times = []
    last_join_time = timerepr.time( "00:00:00" )
    all_pcs = system.get_pcs()
    for char in partners:
        if char in all_pcs:
            if object.has_attribute( char, "joined_party_time" ):
                join_times.append( object.get_attribute( char, "joined_party_time" ) )

    if len( join_times ) > 0:
        last_join_time = max( join_times )

        time_together = system.get_mission_time() - last_join_time
# pour viter des message d'erreur avec l'emploie de system.set_mission_time(). 
# Ce n'est pas important puisque ce n'est que des dialogues entre membres du groupe.
        if time_together < 0:return True
#
        real_time_together = time_together / int( globals.hour_game_to_real )
        
        if timerepr.time( "02:00:00" ) < real_time_together < timerepr.time( "06:00:00" ):
            return True
        else:
            return False

    return False

    

def party_comment():
    """Plays a comment of a party member concerning another party member
    """
    all_pcs = system.get_pcs()
    random.shuffle( all_pcs ) # shuffle all pcs so that we do not get the same order everytime


    for char in all_pcs:
        if char == "MERCENARY_AGNES":
            dialog_partners = [ "MERCENARY_SALLY", "MERCENARY_VASQUEZ" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_SALLY":
                    if mercenary_comment( char, None, "MAGN_COMMENT_C_", 8, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_VASQUEZ":
                    if mercenary_comment( char, None, "MAGN_COMMENT_D_", 10, "party_comment" ) == True:
                        return True
        
        if char == "MERCENARY_KENO":
            dialog_partners = [ "MERCENARY_AGNES", "MERCENARY_CARMEN" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_AGNES":
                    if mercenary_comment( char, None, "MKEN_COMMENT_B_", 8, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_CARMEN":
                    if mercenary_comment( char, None, "MKEN_COMMENT_C_", 5, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_FRANCIS":
            dialog_partners = [ "MERCENARY_STORM", "MERCENARY_KENO" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_STORM":
                    if mercenary_comment( char, None, "MFRA_COMMENT_A_", 5, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_KENO":
                    if mercenary_comment( char, None, "MFRA_COMMENT_B_", 5, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_DEBORAH":
            dialog_partners = [ "MERCENARY_STORM", "MERCENARY_SAND" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_STORM":
                    if mercenary_comment( char, None, "MDEB_COMMENT_C_", 5, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_SAND":
                    if mercenary_comment( char, None, "MDEB_COMMENT_B_", 5, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_CARMEN":
            if "MERCENARY_AGNES" in all_pcs:
                if mercenary_comment( char, None, "MCAR_COMMENT_C_", 10, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_VICTOR":
            if "MERCENARY_KENO" in all_pcs:
                if mercenary_comment( char, None, "MVIC_COMMENT_C_", 5, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_SALLY":
            if "MERCENARY_AGNES" in all_pcs:
                if mercenary_comment( char, None, "MSAL_COMMENT_A_", 5, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_SEVENSOULS":
            if "MERCENARY_MILTON" in all_pcs:
                if mercenary_comment( char, None, "MSEV_COMMENT_B_", 5, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_MILTON":
            if "MERCENARY_SEVENSOULS" in all_pcs:
                if mercenary_comment( char, None, "MMIL_COMMENT_B_", 5, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_SAND":
            dialog_partners = [ "MERCENARY_STORM", "MERCENARY_BUTCH" ]
            random.shuffle( dialog_partners )
            if character.get_body_mode( char ) in [ "LIE", "KNEEL" ]:
                if mercenary_comment( char, None, "MSAN_COMMENT_B_", 5, "party_comment" ) == True:
                    return True

            for partner in dialog_partners:
                if partner == "MERCENARY_STORM":
                    if partner in all_pcs: # if Storm is party member
                        if mercenary_comment( char, None, "MSAN_COMMENT_C_", 6, "party_comment" ) == True:
                            return True
                    else: # if Storm is not a party member
                        if mercenary_comment( char, None, "MSAN_COMMENT_E_", 6, "party_comment" ) == True:
                            return True

                if partner not in all_pcs: continue
                if partner == "MERCENARY_BUTCH":
                    if mercenary_comment( char, None, "MSAN_COMMENT_D_", 5, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_STORM":
            dialog_partners = [ "MERCENARY_SAND", "MERCENARY_BUTCH", "MERCENARY_DEBORAH", "MERCENARY_FRANCIS" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner == "MERCENARY_SAND":
                    if partner not in all_pcs:
                        if mercenary_comment( char, None, "MSTO_COMMENT_B_", 5, "party_comment" ) == True:
                            return True
                if partner not in all_pcs: continue
                if partner == "MERCENARY_BUTCH":
                    if mercenary_comment( char, None, "MSTO_COMMENT_C_", 5, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_FRANCIS":
                    if mercenary_comment( char, None, "MSTO_COMMENT_D_", 5, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_DEBORAH":
                    if mercenary_comment( char, None, "MSTO_COMMENT_E_", 5, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_BUTCH":
            dialog_partners = [ "MERCENARY_SAND", "MERCENARY_STORM" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_SAND":
                    if mercenary_comment( char, None, "MBUT_COMMENT_C_", 7, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_STORM":
                    if mercenary_comment( char, None, "MBUT_COMMENT_D_", 7, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_VASQUEZ":
            dialog_partners = [ "MERCENARY_AGNES", "MERCENARY_HENRY" ]
            random.shuffle( dialog_partners )
            for partner in dialog_partners:
                if partner not in all_pcs: continue
                if partner == "MERCENARY_AGNES":
                    if mercenary_comment( char, None, "MVAS_COMMENT_C_", 5, "party_comment" ) == True:
                        return True
                if partner == "MERCENARY_HENRY":
                    if mercenary_comment( char, None, "MVAS_COMMENT_B_", 10, "party_comment" ) == True:
                        return True

        if char == "MERCENARY_IVANA":
            if "MERCENARY_BUTCH" in all_pcs:
                if mercenary_comment( char, None, "MIVA_COMMENT_C_", 10, "party_comment" ) == True:
                    return True

        if char == "MERCENARY_HENRY":
            # do not change the oder of the partners and the ids
            dialog_partners = [ ( "MERCENARY_AGNES", "B" ), ( "MERCENARY_CARMEN", "C" ), ( "MERCENARY_IVANA", "D" ),
                    ( "MERCENARY_BUTCH", "E" ), ( "MERCENARY_SALLY", "F" ), ( "MERCENARY_SEVENSOULS", "G" ),
                    ( "MERCENARY_MILTON", "H" ), ( "MERCENARY_KENO", "I" ), ( "MERCENARY_FRANCIS", "J" ), 
                    ( "MERCENARY_DEBORAH", "K" ), ( "MERCENARY_VICTOR", "L" ) ]

            random.shuffle( dialog_partners )
            id = ""
            for partner in dialog_partners:
                if partner[ 0 ] not in all_pcs: 
                    continue
                else:
                    id = partner[ 1 ]
                    break

            if id != "":
                if mercenary_comment( char, None, "MHEN_COMMENT_" + id + "_", 1, "party_comment" ) == True:
                    return True

    return False

def make_party_comment():
    """Checks if globals rules apply and then 
    makes a party comment or dialog
    """
    print 'check_global_rules', check_global_rules()
    if check_global_rules():
        if not party_dialog():
            party_comment()
    else:
        if not party_dialog():
            pass


############################
### timer based comments ###
############################
def on_comment_timer():
    """is called cyclic by the programm to check for new comments"""
    return

    if "comments" not in globals.data().__dict__:
        init()

    if (system.in_combat_mode() or
        dialog.is_dialog_active()): return

    PCs = system.get_pcs()
    if "ALTER_EGO" in PCs:
        PCs.remove( "ALTER_EGO" ) # remove ALTER_EGO from list

    if len(PCs) > 0 :
        character_id = random.choice(PCs)

        if character_id in globals.data().comments:
            textbase = globals.data().comments[character_id][0]
            variations = globals.data().comments[character_id][1]

            if len(variations) > 0:
                choice = random.choice(variations)
                comment(character_id, textbase + str(choice))
                globals.data().comments[character_id][1].remove(choice)

### 1.Partymitglied ###
# Wenn X in der Gruppe ist:
# Wenn sich ein Charakter in der Gruppe befindet, der weniger als 30 % seiner LE hat:
# Wenn sich ein Charakter in der Gruppe befindet, der weniger als 10 % seiner LE hat:

### 2. Gegenstand entdeckt ###
# Wenn sie einen H�ndler sieht:
# Wenn er ein besonderes Motorrad sieht:

### 3. Ortschaft betreten ###
# Wenn ein neues Dorf oder eine neue Stadt betreten wird:

### 4. Bewegungsart ###
# Wenn sie sich geduckt oder robbend fortbewegt

### 5. Kampfende ###
# Direkt nachdem ein Kampf gewonnen wurde:

### 6. Explosion ###
# Direkt nach einer Explosion, die in H�rreichweite war:

### 7. zurueckgelegter Weg ###
# Wenn die Party eine Wegstrecke von 1 km ohne Unterbrechung durch das Ansprechen von Personen zu Fu� zur�ckgelegt hat.

### 8. Inventar ###
# Wenn sie ein komplexeres mechanisches oder elektronisches Ger�t ins Inventar bekommt:
# Wenn eine besondere Wurfwaffe ins Inventar kommt:
# Wenn eine besondere schwere Waffe ins Inventar kommt:
# Wenn Nahrung in ihr Inventar kommt:
# Wenn etwas Essbares in ihr Inventar kommt:
# Wenn ein Item ins Inventar kommt, dass man kombinieren kann:
# Wenn eine besondere Spezialwaffe ins Inventar kommt:
# Wenn eine Spezialwaffe in sein Inventar kommt:
# Wenn Fleisch ins Inventar eines Charakters kommt:
# Wenn ein Scharfsch�tzengewehr, dessen Typ er noch nicht hatte, in sein Inventar kommt:
# Wenn Sprengstoff ins Inventar kommt:
