# -*- coding: utf-8 -*-
import sys
import dialog
import character
import object
import system
import debug
#import sound
import animation
import vehicle
import random
import math
from timerepr import *
from daily_routine import *
import globaltext
import skill_tables
import skies

__doc__ = """this module defines functions and constants that should be available everywhere.
each zone module should import this.
"""

LOCAL_USER = -1

#user constants
USER_UNKNOWN = -1
USER_RONNY_KNAUTH = 0
USER_ALEXANDER_MISELER = 1
USER_SEBASTIAN_TUSK = 2
USER_STEFAN_HOFFMANN = 3
USER_DOMINIK_DEICHSEL = 4
USER_JAN_BOEHM = 5
USER_DANIEL_TOEPFER = 6
USER_ADRIAN_KAESTORF = 7
USER_JAN_JORDAN = 8
USER_MARTIN_DIETRICH = 9
USER_MICHAEL_TILLMANN = 10
USER_MALTE_FRITZLER = 11
USER_ALEX_BALDECK = 12
USER_ROGER_WIEGELS = 13
USER_CARSTEN_STARITZ = 14
USER_HEIKO_AULBACH = 15
USER_ANDREAS_BENDER = 16
USER_BENJAMIN_PETER = 17
USER_ANSELM_RINGLEBEN = 18
USER_THOMAS_SCHMIDT = 19
USER_THOMAS_HORN = 20
USER_MIRKO_KOENIG = 21
USER_MICHAEL_MOSEL = 22
USER_PATRICK_BUSCH = 23

OBJECT = 0
PARTY = 1

INACTIVE = 0
ACTIVE = 1

INACTIVE = -1
UNCOMPLETE = 0
COMPLETE = 1
IMPOSSIBLE = 2

ENDDIALOG = -1

MALE = "male"
FEMALE = "female"

DIALOGSMALL = 0
DIALOGLARGE = 1

FLAG_OBJECT_FLEE = 1
FLAG_OBJECT_CHASE = 2
FLAG_OBJECT_EQUIP = 4
FLAG_OBJECT_ATTACK = 8
FLAG_OBJECT_LESSAMMO = 16
FLAG_OBJECT_JOIN = 32
FLAG_OBJECT_GETIN = 64
FLAG_OBJECT_GETOUT = 128

PD_NEUTRAL = 0
PD_ENEMY = 1
PD_FRIEND = 2

NOTAPPROACHABLE = 0
APPROACHABLE = 1

MT_NORMAL = 0 #normal movement
MT_FASTEST = 1 #fastest movement and the char stands up
MT_FAST = 1 #fastest movement and the char stands up
MT_SLOW	= 2 #slow movement
MT_AUTO	= 3 #let distance decide
MT_ATTACKMOVE = 4 #like fastest but the char does not standup if hes lieing

LOOP_PATROL = 1
SINGLE_PATROL = 0

AE_SKIN = 0
AE_FACE = 1
AE_HAIR = 2
AE_BEARD = 3

### Dummy Konstanten ###
DUMMY_1 = 'dmyp_001'
DUMMY_2 = 'dmyp_002'
DUMMY_3 = 'dmyp_003'
DUMMY_4 = 'dmyp_004'
DUMMY_5 = 'dmyp_005'
DUMMY_6 = 'dmyp_006'

### Directions ###
DIR_NORTH     = 0
DIR_NORTH_EAST  = 45
DIR_EAST      = 90
DIR_SOUTH_EAST  = 135
DIR_SOUTH     = 180
DIR_SOUTH_WEST = 225
DIR_WEST     = 270
DIR_NORTH_WEST = 325

## Rogers Zeitumrechnung ##
hour_real_to_game = 1.0/24.0
hour_game_to_real = 24.0

# Söldnerdialoge aktivieren #
ACTIVATE_MERCENARY_COMMENTS = 1

class Lambda:
	"""replacement for lambda
	
since lambda functions can't be pickled (==can't be saved) this replacement
should be used.

example:
	dialog.add_text(text='D_002_T_001', speaker="ALTER_EGO", follower='ENDDIALOG'
		, callback=Lambda('print "test"'))

Lambda can handle statements (print, asignments, asserts, ...) opposing to lambda. 
Lambda accepts parameters:
	my_text = "hello world!"
	f = Lambda('print text', 'text')
	f(my_text)		# prints "hello world!"

local references are not allowed with Lambda:
	def test():
		x=5
		callback=Lambda('print x')	# WRONG! BAD! NO NO!

this is not allowed by the way, because it can't be saved:
def test():
	def helper():
		print "huhu"
	callback=helper
use Lambda instead of nested functions if you want to use it as callbacks.
"""
	def __init__(self, code_string, *parameters):
		"""class constructor"""
		for entry in parameters:
			assert isinstance(entry, str), "parameters need to be strings"

		self.global_key = None
		self.code_string = code_string
		self.params = parameters

		frame = sys._getframe(1) # wir müssen unsere aktion im context unseres callers ausführen
		self.error_string = frame.f_code.co_filename + " (line %d)" % (frame.f_code.co_firstlineno+1)
		for key in sys.modules.keys():
			if sys.modules[key] and (sys.modules[key].__dict__ == frame.f_globals):
				self.global_key = key
		del frame # wichtig: reference cycle verhindern

	def __call__(self, *args, **kw):
		"""makes instances of this class callable"""
		# werden parameter übergeben, die bei __init__ nicht deklariert wurden?
		for key in kw.keys():
			if key not in self.params:
				print "Lambda: parameter '%s' not declared at initialisation." % key

		# locals dict faken, parameter für die lambda funktion ins locals dict eintragen
		arglist = [x for x in args]
		local = sys.modules[self.global_key].__dict__.copy()
		local.update(kw)
		for entry in self.params:
			if entry not in kw:
				assert len(arglist), "not enough parameters given"
				local[entry] = arglist.pop(0)

		eval(compile(self.code_string, "<Lambda: %s>" % self.error_string, "exec"), sys.modules[self.global_key].__dict__, local)


def data(mission_name=None):
	"""access function for the persistence object
	
only informations and data accessed with data() will be saved and loaded properly.
data() gives access to the current zone.
with e.g. data("zone_5.zone_5") you can access other zones.
example:
    data().new_variable = False
    if data().new_variable:
        print "AHA!"
"""
	if not hasattr(sys, "data"):
		sys.data = sys.modules["__main__"].Persistence()
	if mission_name==None:
		mission_name=system.get_act_mission_name()
	if not hasattr(sys.data, mission_name):
		# print "persistence object created: %s" % mission_name
		sys.data.__dict__[mission_name] = sys.modules["__main__"].Persistence()
	return sys.data.__dict__[mission_name]

sys.modules["__main__"].data = data

def was_zone_entered(zone):
	"""returns True if the player was in the given zone

e.g. if was_zone_entered("zone_5.zone_5"):
         print "WOOT! player was in zone_5 already!"
"""
	if not hasattr(sys.data, "zone_entered"):
		sys.data.zone_entered = []
	return zone in sys.data.zone_entered

def is_zone_enabled(zone):
	"""returns True if a zone is enabled
	
enabled means that a zone is shown on the zone map and that the zone can be
entered.
"""
	if not hasattr(sys.data, "zone_enabled"):
		sys.data.zone_enabled = {}
	if not zone in sys.data.zone_enabled:
		sys.data.zone_enabled[zone] = False
	return sys.data.zone_enabled[zone]

def set_zone_enabled(zone, enabled):
	"""sets the zone enabled state
	
enabled means that a zone is shown on the zone map and that the zone can be
entered.
"""
	assert(enabled in (False, True))
	if not hasattr(sys.data, "zone_enabled"):
		sys.data.zone_enabled = {}
	sys.data.zone_enabled[zone] = enabled


all_talents = {
	"close_combat":		["atc","avoid","stamina","baffle","combo"],
	"light_weapons":	["precision","stable_shot","headshot","critical_hit","fast_reload"],
	"heavy_weapons":	["precision","stable_shot","fast_reload","critical_hit","shoulder"],
	"sniper_weapons":	["precision","night_sight","headshot","camouflage","stable_shot"],
	"throw_weapons":	["precision","stable_throw","two_weapon_throw","specialisation_grenades","ninja"],
	"explosives":		["defuse","detect_booby_traps","blast_locks","max_explosive_effect","build_explosives"],
	"survival":			["cook","feign_death","control_animals","best_resting_place","survive"],
	"sneak":			["sneak_fast","crawl_fast","camouflage","night_camouflage","group_camouflage"],
	"pick_pocket":		["inside_pocket","diversion","slink","flatter","excuse"],
	"lock_pick":		["find_lock_traps","specialisation_doors","specialisation_chests","specialisation_safes","lock_trap_trick"],
	"technic":			["repair_weapons","stronger_armor","manipulate_munition","tuning","arm_vehicles"],
	"driving":			["road_battle","driving_tricks","tanks","helicopters","economic_driving"],
	"speech":			["persuade","trade","conversation","offend","increase_moral","entertainment","teach"],
	"medicine":			["specialisation_light_wounds","regeneration","resuscitation","alternative_medicine","psychology"],
}

def helper_get_talents(id, skill):
	import object
	attr = skill+"_"+"talents"
	if not object.has_attribute( id , attr ):
		return []
		# object.set_attribute( id , attr, [] )
	return object.get_attribute( id , attr )

def validate_skill_talent(skill, talent):
	"""check fuction to validate skill and talent names

skill - skill identifier
talent - talent identifier
"""
	if skill in all_talents:
		if talent in all_talents[skill]:
			return True
		else: 
			talent_str = None
			for act_talent in all_talents[skill]:
				if talent_str:
					talent_str+="|"+act_talent
				else:
					talent_str=act_talent
			raise ValueError, "unknown talent\n" \
				+ "availabe talents for '"+skill+"': "+talent_str
	else:
		skill_str = None
		for act_skill in all_talents.keys():
			if skill_str:
				skill_str+="|"+act_skill
			else:
				skill_str=act_skill
		raise ValueError, "unknown skill\n" \
			+ "availabe skills: "+skill_str
	return False

def has_talent(id, skill, talent):
	"""return True if the character has the specified talent

id - id of the character
skill - skill identifier
talent - talent identifier
"""
	if validate_skill_talent(skill, talent):
		char_talents = helper_get_talents( id , skill )
		return talent in char_talents
	return False

def get_talent_count(id, skill):
	"""return the number of assigned talents for the skill

id - id of the character
skill - skill identifier
"""
	char_talents = helper_get_talents( id , skill )
	return len(char_talents)

def get_talent_by_index(id, skill, index):
	"""returns the talent from the talent list with the given index

id - id of the character
skill - skill identifier
index - index of the talent within the talent list of the skill
"""
	char_talents = helper_get_talents( id , skill )
	if index < len(char_talents):
		return char_talents[index] 
	return None

def set_talent_by_index(id, skill, talent, index):
	"""sets the talent with the given index

id - id of the character
skill - skill identifier
talent - talent identifier
index - index of the talent within the talent list of the skill
"""
	if validate_skill_talent(skill, talent):
		char_talents = helper_get_talents( id , skill )
		if index<len(char_talents):
			char_talents[index]=talent
			attr = skill+"_"+"talents"
			object.set_attribute( id , attr, char_talents)
		else:
			raise ValueError, "talent with the index not found"

def remove_talent_by_index(id, skill, index):
	"""removes the talent with the given index

id - id of the character
skill - skill identifier
index - index of the talent within the talent list of the skill
"""
	char_talents = helper_get_talents( id , skill )
	if index<len(char_talents):
		char_talents.remove(char_talents[index])
		attr = skill+"_"+"talents"
		object.set_attribute( id , attr, char_talents)
	else:
		raise ValueError, "talent with the index not found"

def set_talent(id, skill, talent):
	"""sets the specified talent for the character

id - id of the character
skill - skill identifier
talent - talent identifier
"""
	if validate_skill_talent(skill, talent):
		import object
		attr = skill+"_"+"talents"
		if not object.has_attribute( id , attr ):
			object.set_attribute( id , attr, [] )
		char_talents = object.get_attribute( id , attr )
		max_talents_per_skill = 3
		if len(char_talents)<max_talents_per_skill:
			if not talent in char_talents:
				char_talents.append(talent)
			object.set_attribute( id , attr, char_talents)
			char_talents = object.get_attribute( id , attr )
		else: raise ValueError, "maximum count of "+str(max_talents_per_skill)+\
			" talents already assigned for the skill '"+skill+"'"
			
		# update talent effects
		import object_events
		object_events.helper_add_talent_effects(id, character.get_primary_weapon(id), None)

def remove_talent(id, skill, talent):
	"""remove the specified talent for the character

id - id of the character
skill - skill identifier
talent - talent identifier
"""
	if validate_skill_talent(skill, talent):
		import object
		attr = skill+"_"+"talents"
		if not object.has_attribute( id , attr ):
			return
		else:
			char_talents = object.get_attribute( id , attr )
			if talent in char_talents:
				char_talents.remove(talent)
				object.set_attribute( id , attr, char_talents )

def remove_all_talents(id, skill):
	"""remove all talents of the skill

id - id of the character
skill - skill identifier
"""
	import object
	attr = skill+"_"+"talents"
	object.set_attribute( id , attr, [] )

def is_water_repository( item_id ):
	"""returns true if the item can be used
	to store water 

item_id - id of the item where the water should be stored"""

	checklist = [ item_id ]
	item_type = ""

	# append item type to checklist if the item
	# has a type, so that both is checked later
	# to play safe
	if object.has_attribute( item_id, "typeid" ):
		item_type = object.get_attribute( item_id, "typeid" )
		checklist.append( item_type )

	for item in checklist:
		if object.exists( item ):
			if object.has_attribute( item, "water_repository" ):
				if object.get_attribute( item, "water_repository" ) == 1:
					return True

	return False



INVISIBLE 	= 0
VISIBLE 	= 1

ANIMAL_ANTELOPE = 4000
ANIMAL_BEAR = 4001
ANIMAL_TIGER = 4002
ANIMAL_BISON = 4003
ANIMAL_DONKEY = 4004
ANIMAL_DEER = 4005
ANIMAL_COW = 4007
ANIMAL_HORSE = 4008
ANIMAL_LIZARD = 4009
ANIMAL_WARG = 4010
ANIMAL_WILDDOG = 4011
ANIMAL_WOLF = 4012
ANIMAL_PIG = 4013

PT_HUMAN = 0
PT_COMPUTER = 1

#nationality
NAT_AMERICAN = 0
NAT_GERMAN = 1
NAT_ENGLISH = 2
NAT_FRENCH = 3
NAT_SPANISH = 4
NAT_ITALIAN = 5
NAT_SWEDISH = 6
NAT_NORWEGIAN = 7
NAT_BELGIAN = 8
NAT_DUTCH = 9
NAT_HUNGARIAN = 10
NAT_CZECH = 11
NAT_RUSSIAN = 12
NAT_CHINESE = 13
NAT_JAPANESE = 14
NAT_INDIAN = 15
NAT_MEXICAN = 15
NAT_AUSTRALIAN = 17

#last residence
RES_ALABAMA = 0
RES_ARIZONA = 40
RES_CALIFORNIA = 33
RES_NEW_MEXICO = 43
RES_OHIO = 8
RES_TEXAS = 39
RES_WYOMING = 31

#experience of life
EXP_FIREARMS = 0
EXP_MARTIAL_ARTS = 1
EXP_VEHICLES = 2
EXP_SURVIVAL = 3
EXP_CRIME = 4
EXP_HANDCRAFT = 5
EXP_MEDICINE = 6

#bodily features
BOD_NONE = 0
BOD_TALL = 1
BOD_SHORT = 2
BOD_STRONG = 3
BOD_WIRY = 4
BOD_HANDSOME = 5
BOD_UGLY = 6

COMBO_EVERYBODY = 0
COMBO_EASY = 15
COMBO_NORMAL = 30
COMBO_DIFFICULT = 60
COMBO_EXTREME = 75
COMBO_IMPOSSIBLE = 0xFFFFFF


#effect constants
#explosions
FX_EXPLOSION_SHAPE_1 = 1	#parameters: Startzeit|Endzeit|Groesse
FX_EXPLOSION_COMBO_1 = 2	#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel
FX_EXPLOSION_PARTICLE_1 = 3	#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel
FX_EXPLOSION_SMOKE = 4		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel

#fire
FX_FIRE_SHAPE = 7		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit
FX_FIRE_PARTICLE = 8		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit

#smoke
FX_SMOKE = 13			#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe
FX_ROCKET_SMOKE = 16		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe
FX_VEHICLE_SMOKE = 17		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe
FX_AIRPLANE_SMOKE = 18		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe
FX_GAS_CLOUD_SMOKE = 19		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe

FX_DUST = 32				#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe


#flying junk
FX_FLYING_JUNK = 25		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe
FX_EXPLOSION_JUNK = 26		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe

#lights
FX_LIGHT_DYN_SPOT = 27		#parameters: Startzeit|Endzeit|Groesse|Farbe
FX_LIGHT_POINT = 29		#parameters: Startzeit|Endzeit|Groesse|Farbe
FX_LIGHT_GLOW = 30		#parameters: Startzeit|Endzeit|Groesse|Farbe|Typ|Innenwinkel|Aussenwinkel

#weapons
FX_TRACER_AMMUNITION = 40	#parameters: Startzeit|Endzeit|Groesse|StartX|StartY|StartZ|EndX|EndY|EndZ|Farbe
FX_BULLET_HIT = 50		#parameters: Startzeit|Endzeit|Groesse
FX_BULLET_HIT_LS = 51		#parameters: Startzeit|Endzeit|Groesse
FX_SHELL_HIT = 52		#parameters: Startzeit|Endzeit|Groesse
FX_BLOOD_SPLASH = 53		#parameters: Startzeit|Endzeit|Groesse
FX_FLAME_THROWER = 60		#parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit|Farbe|Partikellebenszeit
FX_MUZZLE_FLASH = 61		#parameters: count; frequence in Hz; type of flash (0-7); size of flash; type of stream (0-2); size of stream; fade out time in ms

#default fx settings
				  #parameters: Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit
fx_fire 		= [FX_FIRE_SHAPE,0,0,0.25,0,0]
#fx_smoke 		= [FX_VEHICLE_SMOKE,0,0,1.0,7.0,0.3,0x000000,0.8]

#explsions - parameters: Startzeit|Endzeit|Groesse|Austrittswinkel
fx_explosion 		= [FX_EXPLOSION_COMBO_1,0,1000,1.0,100.0]
fx_explosion_medium	= [FX_EXPLOSION_COMBO_1,0,1000,2.0,100.0]
fx_explosion_big	= [FX_EXPLOSION_COMBO_1,0,1000,3.0,100.0]
fx_explosion_humongous	= [FX_EXPLOSION_COMBO_1,0,1000,6.0,100.0]

fx_shell_hit		= [FX_SHELL_HIT,0,0,1.0]
fx_muzzle_flash		= [FX_MUZZLE_FLASH,3,8.0,1,0.5,1,0.25,50]

#dominikEffekt
fx_smoke_lagerfeuer			= [FX_VEHICLE_SMOKE,0,0,3.0,12.0,0.5,0x000000,0.5]
fx_fire_lagerfeuer_small	= [FX_FIRE_SHAPE,0,0,0.15,0,0]
fx_fire_lagerfeuer 			= [FX_FIRE_SHAPE,0,0,0.35,0,0]
fx_fire_lagerfeuer_big 		= [FX_FIRE_SHAPE,0,0,0.45,0,0]
fx_fire_lagerfeuer_01 		= [FX_FIRE_SHAPE,0,0,0.2,0,0]
fx_fire_lagerfeuer_02 		= [FX_FIRE_SHAPE,0,0,0.19,0,0]
fx_fire_lagerfeuer_03 		= [FX_FIRE_SHAPE,0,0,0.25,0,0]
fx_fire_lagerfeuer_04 		= [FX_FIRE_SHAPE,0,0,0.15,0,0]
#fx_smoke_aschetonne		= [FX_SMOKE,0,0,1.0,0.0,0.9,0x808080,0.5]
#fx_smoke_brennende_tonne	= [FX_VEHICLE_SMOKE,0,0,1.0,7.0,0.3,0x000000,0.5]
fx_fire_brennende_tonne 	= [FX_FIRE_SHAPE,0,0,0.25,0,0]
fx_smoke				 	= [FX_SMOKE,0,0,1.0,90.0,1.0,0x808080,0.5]
#effekt fuer den feuerschlucker
fx_flaming_moe				= [FX_FLAME_THROWER,0,250,5.0,0.0,0.6,0x808080,0.5]
fx_fire_big 			= [FX_FIRE_SHAPE,0,0,1.0,1,0]

### Thomas Effekte ##############################
# fx für Firebasing Big
fx_fire_fire_basin_big_00_1	= [FX_FIRE_SHAPE,0,0,0.3,0,0]
fx_fire_fire_basin_big_00_2	= [FX_FIRE_SHAPE,0,0,0.15,0,0]
fx_fire_fire_basin_big_00_3	= [FX_FIRE_SHAPE,0,0,0.15,0,0]
fx_fire_fire_basin_big_00_4	= [FX_FIRE_SHAPE,0,0,0.15,0,0]
fx_fire_fire_basin_big_00_5	= [FX_FIRE_SHAPE,0,0,0.15,0,0]
fx_fire_fire_basin_big_00_6	= [FX_FIRE_SHAPE,0,0,0.7,0,0]
fx_fire_fire_basin_big_00_7	= [FX_FIRE_SHAPE,0,0,0.4,0,0]
fx_fire_fire_basin_big_00_8	= [FX_FIRE_SHAPE,0,0,0.4,0,0]
fx_fire_fire_basin_big_00_9	= [FX_FIRE_SHAPE,0,0,0.4,0,0]
fx_fire_fire_basin_big_00_10	= [FX_FIRE_SHAPE,0,0,0.4,0,0]

# fx für Firebasing Small
fx_fire_basin_small		= [FX_FIRE_SHAPE,0,0,0.1,0,0]

# fx für Pilar_1
fx_fire_pilar_1			= [FX_FIRE_SHAPE,0,0,0.15,0,0]

# fx für Pilar_2
fx_fire_pilar_2			= [FX_FIRE_SHAPE,0,0,0.10,0,0]

# fx für Excavator
fx_smoce_rock_1			= [FX_ROCKET_SMOKE,	1000,18 * 1000,4.0,0.0,10.0,0,1.0]
fx_smoce_rock_2			= [FX_ROCKET_SMOKE,	500,18 * 1000,2.0,0.0,10.0,0,1.0]
fx_smoce_rock_3			= [FX_ROCKET_SMOKE,	500,18 * 1000,1.0,0.0,10.0,0,1.0]
fx_smoce_rock_4			= [FX_AIRPLANE_SMOKE,	500,12 * 1000,0.2,0.0,10.0,0,1.0]
#######################################################

# fackel fx				Startzeit|Endzeit|Groesse|Austrittswinkel|Austrittsgeschwindigkeit
fx_torch_fire = [FX_FIRE_SHAPE,0,8000,0.05,180,0.5]
fx_lock_trap = [FX_FIRE_SHAPE,0,5000,0.05,180,0.5]

#app states
APP_UNINITIALIZED = 0
APP_START = 1
APP_EDITOR = 2
APP_GAME = 3
APP_LOAD = 4

FADE_IN = 0
FADE_OUT = 1

#ui visibility status
UIVIS_INVISIBLE = 0
UIVIS_VISIBLE = 1

# ui elements for system.set_uielement_state
UI_ELEMENT_MINIMAP = 0
UI_ELEMENT_CHARACTER_LIST = 1
UI_ELEMENT_CHARACTER_IS_TIRED = 2
UI_ELEMENT_RESTPANEL = 3

#cutscene modes
CINEMATIC = 0
INTERACTIVE = 1

#information panel modes
IPM_NOTE = 0
IPM_BOOK = 1

# fire position/animation
FPA_PISTOL_SINGLE = 0
FPA_PISTOL_VOLLEY = 1
FPA_SHOULDER_SINGLE = 2
FPA_SHOULDER_VOLLEY = 3
FPA_SHOULDER_BASED = 4
FPA_SNIPER_SINGLE = 5
FPA_THROW_GRENADE = 6
FPA_CLOSE_COMBAT_BOXING = 7
FPA_CLOSE_COMBAT_BATON = 8
FPA_CLOSE_COMBAT_THRUSTING = 9
FPA_CLOSE_COMBAT_CHAIN_SAW = 10
FPA_THROW_KNIFE = 11

AUDIO_STREAM_FADE_IN = 0
AUDIO_STREAM_FADE_OUT = 1


def get_formated_date(y,m,d, format):
	import globaltext
	if hasattr(globaltext, "format_date"):
		return globaltext.format_date(y,m,d, format)
	else:
		raise Exception, "missing method in active language"
	return ""

def get_formated_time(time, format):
	import globaltext
	s = m = h = d = 0
	s = (time / 1000) % 60
	m = (time / (1000 * 60)) % 60
	h = (time / (1000 * 60 * 60)) % 24
	d = time / (1000 * 60 * 60 * 24)
	if hasattr(globaltext, "format_time"):
		return globaltext.format_time(d,h,m,s, format)
	else:
		raise Exception, "missing method in active language"
	return ""

import water
import sounds
